import MuseScore 3.0
import QtQuick 2.0

MuseScore {
version: "1.0";
description: "This demo plugin adds new chords\n\n" +
                "Comments, feedbacks, report bugs, ask for new features, contribute:\n" +
                "https://github.com/rousselmanu/msc_plugins/";
menuPath: "Plugins.Chords." + qsTr("Add Chord Demo")
    


function chordnotes(chord, tpc2note){
    var tpc = chord[4];
    var s = tpc2note[tpc+1][1];
    for (var i=5; i < 8; i++){
        tpc = chord[i];
        s += ", " + tpc2note[tpc+1][1];
    }
    return s;
}

function correctCase(s){
    var h = s.substr(0,1);
    s = h.toUpperCase() + s.substr(1);
    return s;
}

// create and return a new Note element with given pitch, tpc
function createNote(pitch, tpc){
    ////console.log("pitch = ", pitch, "tpc = ", tpc)
    var note = newElement(Element.NOTE);
    note.pitch = pitch;
    note.tpc = tpc;
    note.tpc1 = tpc;
    note.tpc2 = tpc;
    return note;
}

function setCursorToTime(cursor, time){
    cursor.rewind(0);
    while (cursor.segment) { 
        var current_time = cursor.tick;
        if(current_time>=time){
            return true;
        }
        cursor.next();
    }
    cursor.rewind(0);
    return false;
}

function abs(c){
    if (c < 0){
        c = -c;
    }
    return c;
}

function findstr(s,chords){
    for(var i=0; i < chords.length;i++){
        if(chords[i][0] == s) return 1;
    }
    return 0;
}

//adds chord at current position. chords is an array with pitch of notes.
function addChord(cursor, chords, duration){ 
    if(chords.length==0) return -1;
    var cur_time=cursor.tick;
    //console.log("cur_time = ", cur_time);
    cursor.setDuration(duration, 1920);
    cursor.addNote(chords[0]); //add 1st note
    
    var next_time=cursor.tick;
    //console.log("next_time = ", next_time);
    setCursorToTime(cursor, cur_time); //rewind to this note
    var note = cursor.element.notes[0];
    note.tpc = chords[4];
    
    var chord = cursor.element; //get the chord created when 1st note was inserted
            
    for(var i=1; i<4; i++){
        var note = createNote(chords[i], chords[i+4]);
        //console.log("note.pitch = ", note.pitch, "note.tpc = ", note.tpc);
        note.tpc = chords[i+4];
        chord.add(note); //add notes to the chord
        chord.notes[i].tpc = chords[i+4];
    }
    
    setCursorToTime(cursor, next_time);
    return 0;
}
function collectchordnames(segment, chordnames,count){
    while (segment) {
        var aCount = 0;
        var annotation = segment.annotations[aCount];

        while (annotation) {
            if (annotation.type == Element.HARMONY) {
                var harmony =  annotation;
                var str = harmony.text;
                if (str != "")
                    str = correctCase(str);
                var r = str.indexOf("Maj");
                if(str.substr(str.length-3,3)== "Maj"){
                    str = str.substr(0,str.length-3)
                    //console.log('Maj string = ' + str);
                }
                chordnames[count][0] = str;
                chordnames[count][1] = segment.tick;
                
                // avoid spurious empty harmonies
                if (chordnames[count][0] != ""){
                    count++;
                }					    
            }
            annotation = segment.annotations[++aCount];
        }
        segment = segment.next;
    }
    return count;
}

    onRun: {
        if (typeof curScore === 'undefined') {
            Qt.quit();
        }

        var cursor = curScore.newCursor(),
                startStaff,
                endStaff,
                endTick,
                fullScore = false;
                
        cursor.rewind(1);
        
       
        if (!cursor.segment) { // no selection
            fullScore = true;
            startStaff = 0; // start with 1st staff
            endStaff = curScore.nstaves - 1; // and end with last
        } else {
            startStaff = cursor.staffIdx;
            cursor.rewind(2);
            if (cursor.tick === 0) {
                // this happens when the selection includes
                // the last measure of the score.
                // rewind(2) goes behind the last segment (where
                // there's none) and sets tick=0
                endTick = curScore.lastSegment.tick + 1;
            } else {
                endTick = cursor.tick;
            }
            endStaff = cursor.staffIdx;
        }
          
        cursor.rewind(1); // beginning of selection
        if (fullScore) { // no selection
            cursor.rewind(0); // beginning of score
        }
        cursor.voice = 0;
        
		cursor.staffIdx = 1;

        // chords[2] & [3] no longer used
        // chords[0]: name
        // chords[1]: pitches lo-hi & tpc lo-hi
        // chords[2]: occurences of this voicing (not used anymore)
        // chords[3]: instances of this chord name (not used anymore)
        // constructed in Plugins\Harmonyvoicings\Writechords.xlsm
var chords = [
["A",[56,57,61,64,22,17,21,18],1,1], //g#,a,c#,e
["A#7",[56,60,62,67,22,14,16,15],1,1], //g#,c,d,g
["A#7b13",[56,60,62,66,22,14,16,20],1,1],   //g#,c,d,f#
["A#7sus",[53,56,58,63,13,22,24,23],1,1], //f,g#,a#,d#
["A#m7b5",[56,58,61,64,22,24,21,18],1,1], //g#,a#,c#,e
["A6/9",[54,59,61,64,20,19,21,18],1,1], //f#,b,c#,e
["A7",[55,59,61,66,15,19,21,20],1,1], //g,b,c#,f#
["A7#11",[57,61,63,67,17,21,23,15],1,1], //a,c#,d#,g
["A7#11/C#",[49,51,55,57,21,11,15,17],1,1], //c#,eb,g,a
["A7#9",[49,52,55,60,21,18,15,14],1,1], //c#,e,g,c
["A7/C#",[49,54,55,59,21,20,15,19],1,1], //c#,f#,g,b
["A7/E",[52,55,57,61,18,15,17,21],1,1], //e,g,a,c#
["A7alt",[55,60,61,65,15,14,9,13],1,1], //g,c,db,f
["A7b13",[55,59,61,65,15,19,21,13],1,1], //g,b,c#,f
["A7b9",[49,54,55,58,21,20,15,12],1,1], //c#,f#,g,bb
["A7b9",[55,58,61,66,15,12,21,20],1,1], //g,bb,c#,f#
["A7sus",[55,57,62,64,15,17,16,18],1,1], //g,a,d,e
["A7susb9",[52,57,58,62,18,17,12,16],1,1], //e,a,bb,d
["Ab",[55,56,60,63,15,10,14,11],1,1], //g,ab,c,eb
["Ab/Ab",[56,60,63,67,10,14,11,15],1,1],   //ab,c,eb,g
["Ab/Eb",[51,55,56,60,11,15,10,14],1,1], //eb,g,ab,c
["Ab6/9",[53,58,60,63,13,12,14,11],1,1], //f,bb,c,eb
["Ab7",[54,58,60,65,8,12,14,13],1,1], //gb,bb,c,f
["Ab7#11",[56,60,62,66,10,14,16,8],1,1], //ab,c,d,gb
["Ab7#11/Gb",[54,56,60,62,8,10,14,16],1,1], //gb,ab,c,d
["Ab7#9",[48,54,56,59,14,8,10,19],1,1], //c,gb,ab,b
["Ab7/C",[48,53,54,58,14,13,8,12],1,1], //c,f,gb,bb
["Ab7/Eb",[51,54,56,60,11,8,10,14],1,1], //eb,gb,ab,c
["Ab7/Gb",[54,58,60,65,8,12,14,13],1,1], //gb,bb,c,f
["Ab7alt",[54,59,60,64,20,19,14,18],1,1], //f#,b,c,e
["Ab7b13",[54,58,60,64,8,12,14,18],1,1], //gb,bb,c,e
["Ab7b9",[48,53,54,57,14,13,8,17],1,1], //c,f,gb,a
["Ab7b9",[54,57,60,65,8,17,14,13],1,1], //gb,a,c,f
["Ab7sus",[54,56,61,63,8,10,9,11],1,1], //gb,ab,db,eb
["Ab7sus/Gb",[54,56,61,63,8,10,9,11],1,1], //gb,ab,db,eb
["Ab7susb9",[51,56,57,61,11,10,17,9],1,1], //eb,ab,a,db
["Abdim",[50,53,56,59,16,13,10,19],1,1], //d,f,ab,b
["Abdim",[53,56,59,62,13,10,19,16],1,1], //f,ab,b,d
["Abdim",[56,59,62,65,10,19,16,13],1,1], //ab,b,d,f
["Abdim",[59,62,65,68,19,16,13,10],1,1], //b,d,f,ab
["Abm(6/9)",[53,58,59,63,13,12,19,11],1,1], //f,bb,b,eb
["Abm6",[56,59,63,65,10,19,11,13],1,1], //ab,b,eb,f
["Abm7",[54,58,59,63,8,12,7,11],1,1], //gb,bb,cb,eb
["Abm7#5",[54,56,59,64,8,10,19,18],1,1], //gb,ab,b,e
["Abm7/Eb",[51,54,56,59,11,8,10,19],1,1], //eb,gb,ab,b
["Abm7/Gb",[54,58,59,63,8,12,7,11],1,1], //gb,bb,cb,eb
["Abm7b5",[54,56,59,62,8,10,19,16],1,1], //gb,ab,b,d
["Abm7b5/D",[50,54,56,59,16,8,10,19],1,1], //d,gb,ab,b
["AbMaj#11",[56,60,62,67,10,14,16,15],1,1], //ab,c,d,g
["AbMaj#5",[52,55,56,60,18,15,10,14],1,1], //e,g,ab,c
["AbmMaj7",[55,56,59,63,15,10,19,11],1,1], //g,ab,b,eb
["Adim",[48,51,54,57,14,11,20,17],1,1], //c,eb,f#,a
["Adim",[51,54,57,60,11,20,17,14],1,1], //eb,f#,a,c
["Adim",[54,57,60,63,20,17,14,11],1,1], //f#,a,c,eb
["Adim",[57,60,63,66,17,14,11,20],1,1], //a,c,eb,f#
["Am(6/9)",[54,59,60,64,20,19,14,18],1,1], //f#,b,c,e
["Am6",[57,60,64,66,17,14,18,20],1,1], //a,c,e,f#
["Am7",[55,59,60,64,15,19,14,18],1,1], //g,b,c,e
["Am7#5",[55,57,60,65,15,17,14,13],1,1], //g,a,c,f
["Am7/G",[55,59,60,64,15,19,14,18],1,1], //g,b,c,e
["Am7b5",[55,57,60,63,15,17,14,11],1,1], //g,a,c,eb
["Am7b5/A",[57,60,63,67,17,14,11,15],1,1],   //a,c,eb,g
["Am7b5/C",[48,51,55,57,14,11,15,17],1,1], //c,eb,g,a
["Am7b5/Eb",[51,55,57,60,11,15,17,14],1,1], //eb,g,a,c
["AMaj#11",[56,57,61,63,22,17,21,23],1,1], //g#,a,c#,d#
["AMaj#5",[56,57,61,65,22,17,21,25],1,1], //g#,a,c#,e#
["AmMaj7",[56,57,60,64,22,17,14,18],1,1], //g#,a,c,e
["B",[47,51,54,58,19,23,20,24],1,1], //b,d#,f#,a#
["B",[51,54,58,59,23,20,24,19],1,1], //d#,f#,a#,b
["B",[58,59,63,66,24,19,23,20],1,1], //a#,b,d#,f#
["B6/9",[51,54,56,61,23,20,22,21],1,1], //d#,f#,g#,c#
["B7",[51,56,57,61,23,22,17,21],1,1], //d#,g#,a,c#
["B7#11",[53,57,59,63,13,17,19,23],1,1], //f,a,b,d#
["B7#11/D#",[51,53,57,59,23,13,17,19],1,1], //d#,f,a,b
["B7#9",[51,57,59,62,23,17,19,16],1,1], //d#,a,b,d
["B7/D#",[51,54,57,59,23,20,17,19],1,1], //d#,f#,a,b
["B7/F#",[54,57,59,63,20,17,19,23],1,1], //f#,a,b,d#
["B7alt",[51,55,57,62,11,15,17,16],1,1], //eb,g,a,d
["B7b13",[51,55,57,61,23,15,17,21],1,1], //d#,g,a,c#
["B7b13",[57,61,63,67,17,21,23,15],1,1], //a,c#,d#,g
["B7b9",[51,56,57,60,23,22,17,14],1,1], //d#,g#,a,c
["B7sus",[52,54,57,59,18,20,17,19],1,1], //e,f#,a,b
["B7susb9",[54,59,60,64,20,19,14,18],1,1], //f#,b,c,e
["Bb",[57,58,62,65,17,12,16,13],1,1], //a,bb,d,f
["Bb/D",[50,53,57,58,16,13,17,12],1,1], //d,f,a,bb
["Bb/F",[53,57,58,62,13,17,12,16],1,1], //f,a,bb,d
["Bb6/9",[50,53,55,60,16,13,15,14],1,1], //d,f,g,c
["Bb7",[50,55,56,60,16,15,10,14],1,1], //d,g,ab,c
["Bb7",[56,60,62,67,10,14,16,15],1,1], //ab,c,d,g
["Bb7#11",[52,56,58,62,18,10,12,16],1,1], //e,ab,bb,d
["Bb7#9",[50,56,58,61,16,10,12,9],1,1], //d,ab,bb,db
["Bb7/Ab",[56,60,62,67,10,14,16,15],1,1], //ab,c,d,g
["Bb7/F",[53,56,58,62,13,10,12,16],1,1], //f,ab,bb,d
["Bb7alt",[50,54,56,61,16,20,22,21],1,1], //d,f#,g#,c#
["Bb7alt",[56,61,62,66,22,21,16,20],1,1], //g#,c#,d,f#
["Bb7b13",[56,60,62,66,10,14,16,8],1,1], //ab,c,d,gb
["Bb7b9",[50,55,56,59,16,15,10,19],1,1], //d,g,ab,b
["Bb7b9",[56,59,62,67,10,19,16,15],1,1], //ab,b,d,g
["Bb7sus",[51,53,56,58,11,13,10,12],1,1], //eb,f,ab,bb
["Bb7sus",[53,56,58,63,13,10,12,11],1,1], //f,ab,bb,eb
["Bb7sus/Ab",[56,58,63,65,10,12,11,13],1,1], //ab,bb,eb,f
["Bb7sus/Eb",[51,53,56,58,11,13,10,12],1,1], //eb,f,ab,bb
["Bb7susb9",[53,58,59,63,13,12,19,11],1,1], //f,bb,b,eb
["Bbdim",[49,52,55,58,21,18,15,12],1,1], //c#,e,g,bb
["Bbdim",[52,55,58,61,18,15,12,21],1,1], //e,g,bb,c#
["Bbdim",[55,58,61,64,15,12,21,18],1,1], //g,bb,c#,e
["Bbdim",[58,61,64,67,12,21,18,15],1,1], //bb,c#,e,g
["Bbm(6/9)",[49,55,58,60,9,15,12,14],1,1], //db,g,bb,c
["Bbm6",[58,61,65,67,12,9,13,15],1,1], //bb,db,f,g
["Bbm7",[56,60,61,65,10,14,9,13],1,1], //ab,c,db,f
["Bbm7#5",[56,60,61,66,10,14,9,20],1,1], //ab,c,db,f#
["Bbm7/Ab",[56,60,61,65,10,14,9,13],1,1], //ab,c,db,f
["Bbm7b5",[56,58,61,64,10,12,9,18],1,1], //ab,bb,db,e
["Bbm7b5/Db",[49,52,56,58,9,18,10,12],1,1], //db,e,ab,bb
["Bbm7b5/E",[52,56,58,61,18,10,12,9],1,1], //e,ab,bb,db
["BbMaj#11",[52,57,58,62,18,17,12,16],1,1], //e,a,bb,d
["BbMaj#11",[57,58,62,64,17,12,16,18],1,1], //a,bb,d,e
["BbMaj#5",[54,57,58,62,20,17,12,16],1,1], //f#,a,bb,d
["BbmMaj7",[49,53,57,60,9,13,17,14],1,1], //db,f,a,c
["BbmMaj7",[57,58,61,65,17,12,9,13],1,1], //a,bb,db,f
["Bdim",[50,53,56,59,16,13,10,19],1,1], //d,f,ab,b
["Bdim",[53,56,59,62,13,10,19,16],1,1], //f,ab,b,d
["Bdim",[56,59,62,65,10,19,16,13],1,1], //ab,b,d,f
["Bdim",[59,62,65,68,19,16,13,10],1,1], //b,d,f,ab
["Bm(6/9)",[50,56,59,61,16,22,19,21],1,1], //d,g#,b,c#
["Bm6",[59,62,66,68,19,16,20,22],1,1], //b,d,f#,g#
["Bm7",[57,61,62,66,17,21,16,20],1,1], //a,c#,d,f#
["Bm7#5",[50,55,57,59,16,15,17,19],1,1], //d,g,a,b
["Bm7/F#",[54,57,59,62,20,17,19,16],1,1], //f#,a,b,d
["Bm7b5",[57,59,62,65,17,19,16,13],1,1], //a,b,d,f
["Bm7b5/F",[53,57,59,62,13,17,19,16],1,1], //f,a,b,d
["BMaj#11",[53,58,59,63,13,24,19,23],1,1], //f,a#,b,d#
["BMaj#5",[55,58,59,63,15,24,19,23],1,1], //g,a#,b,d#
["BmMaj7",[58,59,62,66,24,19,16,20],1,1], //a#,b,d,f#
["C",[48,52,55,59,14,18,15,19],1,1], //c,e,g,b
["C",[52,55,57,62,18,15,17,16],1,1], //e,g,a,d
["C",[59,60,64,67,19,14,18,15],1,1], //b,c,e,g
["C#7",[53,58,59,63,13,24,19,23],1,1], //f,a#,b,d#
["C#7#11",[49,53,55,59,21,13,15,19],1,1], //c#,f,g,b
["C#dim",[49,52,55,58,21,18,15,12],1,1], //c#,e,g,bb
["C#dim",[52,55,58,61,18,15,12,21],1,1], //e,g,bb,c#
["C#dim",[55,58,61,64,15,12,21,18],1,1], //g,bb,c#,e
["C#dim",[58,61,64,67,12,21,18,15],1,1], //bb,c#,e,g
["C#m7",[52,56,59,63,18,22,19,23],1,1], //e,g#,b,d#
["C#m7#5",[49,52,57,59,21,18,17,19],1,1], //c#,e,a,b
["C#m7b5",[49,52,55,59,21,18,15,19],1,1], //c#,e,g,b
["C#m7b5",[49,54,55,59,21,20,15,19],1,1], //c#,f#,g,b
["C#mMaj7",[52,56,60,63,18,22,14,23],1,1], //e,g#,c,d#
["C/B",[59,60,64,67,19,14,18,15],1,1], //b,c,e,g
["C/C",[48,52,55,59,14,18,15,19],1,1], //c,e,g,b
["C/E",[52,55,59,60,18,15,19,14],1,1], //e,g,b,c
["C/G",[55,59,60,64,15,19,14,18],1,1], //g,b,c,e
["C6/9",[52,55,57,62,18,15,17,16],1,1], //e,g,a,d
["C7",[52,57,58,62,18,17,12,16],1,1], //e,a,bb,d
["C7#11",[52,54,58,62,18,20,12,16],1,1], //e,f#,bb,d
["C7#11",[54,58,60,64,20,12,14,18],1,1], //f#,bb,c,e
["C7#9",[52,58,60,63,18,12,14,11],1,1], //e,bb,c,eb
["C7/Bb",[58,60,64,67,12,14,18,15],1,1], //bb,c,e,g
["C7/G",[55,58,60,64,15,12,14,18],1,1], //g,bb,c,e
["C7alt",[52,56,58,63,18,10,12,11],1,1], //e,ab,bb,eb
["C7b13",[52,56,58,62,18,10,12,16],1,1], //e,ab,bb,d
["C7b13",[48,52,56,58,14,18,10,12],1,1], //c,e,ab,bb
["C7b9",[52,57,58,61,18,17,12,9],1,1], //e,a,bb,db
["C7sus",[53,55,58,60,13,15,12,14],1,1], //f,g,bb,c
["C7sus/F",[53,55,58,60,13,15,12,14],1,1], //f,g,bb,c
["C7susb9",[49,53,55,60,9,13,15,14],1,1], //db,f,g,c
["Cdim",[48,51,54,57,14,11,20,17],1,1], //c,eb,f#,a
["Cdim",[51,54,57,60,11,20,17,14],1,1], //eb,f#,a,c
["Cdim",[54,57,60,63,20,17,14,11],1,1], //f#,a,c,eb
["Cdim",[57,60,63,66,17,14,11,20],1,1], //a,c,eb,f#
["Cm(6/9)",[51,57,60,62,11,17,14,16],1,1], //eb,a,c,d
["Cm6",[51,55,57,60,11,15,17,14],1,1], //eb,g,a,c
["Cm7",[48,51,55,58,14,11,15,12],1,1], //c,eb,g,bb
["Cm7",[51,55,58,62,11,15,12,16],1,1], //eb,g,bb,d
["Cm7",[58,62,63,67,12,16,11,15],1,1], //bb,d,eb,g
["Cm7#5",[48,51,56,58,14,11,22,12],1,1],   //c,eb,g#,bb
["Cm7/Bb",[58,60,63,67,12,14,11,15],1,1], //bb,c,eb,g
["Cm7/G",[55,58,60,63,15,12,14,11],1,1], //g,bb,c,eb
["Cm7b5",[48,51,54,58,14,11,8,12],1,1], //c,eb,gb,bb
["Cm7b5",[48,53,54,58,14,13,8,12],1,1], //c,f,gb,bb
["Cm7b5/Eb",[51,54,58,60,11,8,12,14],1,1], //eb,gb,bb,c
["Cm7b5/Gb",[54,58,60,63,8,12,14,11],1,1], //gb,bb,c,eb
["CMaj#11",[54,59,60,64,20,19,14,18],1,1], //f#,b,c,e
["CMaj#5",[48,52,56,59,14,18,22,19],1,1], //c,e,g#,b
["CmMaj7",[48,51,55,59,14,11,15,19],1,1], //c,eb,g,b
["CmMaj7",[51,55,59,62,11,15,19,16],1,1], //eb,g,b,d
["CmMaj7/B",[59,60,63,67,19,14,11,15],1,1], //b,c,eb,g
["D",[49,50,54,57,21,16,20,17],1,1], //c#,d,f#,a
["D",[50,54,57,61,16,20,17,21],1,1], //d,f#,a,c#
["D",[54,57,61,62,20,17,21,16],1,1], //f#,a,c#,d
["D/F#",[54,57,61,62,20,17,21,16],1,1], //f#,a,c#,d
["D#7",[55,60,61,65,15,14,21,13],1,1], //g,c,c#,f
["D#7b9",[55,60,61,64,15,14,9,18],1,1], //g,c,db,e
["D#7#11",[51,55,57,61,23,15,17,21],1,1], //d#,g,a,c#
["D#7b13",[55,59,61,65,15,19,21,13],1,1], //g,b,c#,f
["D#m7b5",[51,56,57,61,23,22,17,21],1,1], //d#,g#,a,c#
["D6/9",[54,57,59,64,20,17,19,18],1,1], //f#,a,b,e
["D7",[54,59,60,64,20,19,14,18],1,1], //f#,b,c,e
["D7#11",[50,54,56,60,16,20,22,14],1,1], //d,f#,g#,c
["D7#11/D",[50,54,56,60,16,20,22,14],1,1], //d,f#,g#,c
["D7#11/F#",[54,56,60,62,20,22,14,16],1,1], //f#,g#,c,d
["D7#9",[54,60,62,65,20,14,16,13],1,1], //f#,c,d,f
["D7/A",[57,60,62,66,17,14,16,20],1,1], //a,c,d,f#
["D7/D",[50,54,57,60,16,20,17,14],1,1], //d,f#,a,c
["D7alt",[54,58,60,65,8,12,14,13],1,1], //gb,bb,c,f
["D7alt/C",[48,53,54,58,14,13,8,12],1,1], //c,f,gb,bb
["D7b13",[54,58,60,64,20,12,14,18],1,1], //f#,bb,c,e
["D7b9",[54,59,60,63,20,19,14,11],1,1], //f#,b,c,eb
["D7b9",[48,51,54,59,14,11,20,19],1,1], //c,eb,f#,b
["D7sus",[55,57,60,62,15,17,14,16],1,1], //g,a,c,d
["D7susb9",[51,55,57,62,11,15,17,16],1,1], //eb,g,a,d
["Db",[53,56,60,61,13,10,14,9],1,1], //f,ab,c,db
["Db/Ab",[56,60,61,65,10,14,9,13],1,1], //ab,c,db,f
["Db/C",[48,49,53,56,14,9,13,10],1,1], //c,db,f,ab
["Db/Db",[49,53,56,60,9,13,10,14],1,1], //db,f,ab,c
["Db/F",[53,56,60,61,13,10,14,9],1,1], //f,ab,c,db
["Db6/9",[53,56,58,63,13,10,12,11],1,1], //f,ab,bb,eb
["Db7",[53,58,59,63,13,12,7,11],1,1], //f,bb,cb,eb
["Db7#11",[49,53,55,59,9,13,15,19],1,1], //db,f,g,b
["Db7#11/F",[53,55,59,61,13,15,19,9],1,1], //f,g,b,db
["Db7#9",[53,59,61,64,13,19,9,18],1,1], //f,b,db,e
["Db7alt",[53,57,59,64,13,17,19,18],1,1], //f,a,b,e
["Db7b13",[49,53,57,59,9,13,17,19],1,1], //db,f,a,b
["Db7b13",[53,57,59,63,13,17,19,11],1,1], //f,a,b,eb
["Db7b9",[53,58,59,62,13,12,7,16],1,1], //f,bb,cb,d
["Db7sus",[54,56,59,61,8,10,19,9],1,1], //gb,ab,b,db
["Db7susb9",[50,54,56,61,16,8,10,9],1,1], //d,gb,ab,db
["Dbm(6/9)",[52,58,61,63,18,12,9,11],1,1], //e,bb,db,eb
["Dbm6",[52,56,58,61,18,10,12,9],1,1], //e,ab,bb,db
["Dbm7",[52,56,59,63,18,10,19,11],1,1], //e,ab,b,eb
["Dbm7",[56,59,61,64,10,19,9,18],1,1], //ab,b,db,e
["Dbm7b5",[49,54,55,59,9,8,15,19],1,1], //db,gb,g,b
["Dbm7b5/G",[55,59,61,64,15,19,9,18],1,1], //g,b,db,e
["DbMaj#11",[49,53,55,60,9,13,15,14],1,1], //db,f,g,c
["DbMaj#5",[49,53,57,60,9,13,17,14],1,1], //db,f,a,c
["DbmMaj7",[49,52,56,60,9,18,10,14],1,1], //db,e,ab,c
["DbmMaj7",[52,56,60,63,18,10,14,11],1,1], //e,ab,c,eb
["DbmMaj7/Ab",[56,60,61,64,10,14,9,18],1,1], //ab,c,db,e
["Ddim",[50,53,56,59,16,13,10,19],1,1], //d,f,ab,b
["Ddim",[53,56,59,62,13,10,19,16],1,1], //f,ab,b,d
["Ddim",[56,59,62,65,10,19,16,13],1,1], //ab,b,d,f
["Ddim",[59,62,65,68,19,16,13,10],1,1], //b,d,f,ab
["Ddim/C",[48,50,53,59,14,16,13,19],1,1], //c,d,f,b
["Dm(6/9)",[53,59,62,64,13,19,16,18],1,1], //f,b,d,e
["Dm6",[53,57,59,62,13,17,19,16],1,1], //f,a,b,d
["Dm7",[53,57,60,64,13,17,14,18],1,1], //f,a,c,e
["Dm7#5",[50,53,58,60,16,13,12,14],1,1], //d,f,bb,c
["Dm7/A",[57,60,62,65,17,14,16,13],1,1], //a,c,d,f
["Dm7/C",[48,50,53,57,14,16,13,17],1,1], //c,d,f,a
["Dm7/D",[50,53,57,60,16,13,17,14],1,1], //d,f,a,c
["Dm7b5",[50,55,56,60,16,15,10,14],1,1], //d,g,ab,c
["Dm7b5/Ab",[56,60,62,65,10,14,16,13],1,1], //ab,c,d,f
["Dm7b5/F",[53,56,60,62,13,10,14,16],1,1], //f,ab,c,d
["DMaj#11",[50,54,56,61,16,20,22,21],1,1], //d,f#,g#,c#
["DMaj#5",[50,54,58,61,16,20,24,21],1,1], //d,f#,a#,c#
["DmMaj7",[49,50,53,57,21,16,13,17],1,1], //c#,d,f,a
["DmMaj7",[53,57,61,64,13,17,21,18],1,1], //f,a,c#,e
["DmMaj7/C#",[49,50,53,57,21,16,13,17],1,1], //c#,d,f,a
["E",[51,52,56,59,23,18,22,19],1,1], //d#,e,g#,b
["E",[56,59,63,64,22,19,23,18],1,1], //g#,b,d#,e
["E6/9",[56,59,61,66,22,19,21,20],1,1], //g#,b,c#,f#
["E7",[56,61,62,66,22,21,16,20],1,1], //g#,c#,d,f#
["E7#11",[52,56,58,62,18,22,24,16],1,1], //e,g#,a#,d
["E7#11/E",[52,56,58,62,18,22,24,16],1,1], //e,g#,a#,d
["E7#9",[56,62,64,67,22,16,18,15],1,1], //g#,d,e,g
["E7/B",[59,62,64,68,19,16,18,22],1,1], //b,d,e,g#
["E7/D",[50,54,56,61,16,20,22,21],1,1], //d,f#,g#,c#
["E7alt",[50,55,56,60,16,15,10,14],1,1], //d,g,ab,c
["E7alt",[56,60,62,67,10,14,16,15],1,1], //ab,c,d,g
["E7b13",[56,60,62,66,22,14,16,20],1,1], //g#,c,d,f#
["E7b9",[50,53,56,61,16,13,22,21],1,1], //d,f,g#,c#
["E7b9",[56,61,62,65,22,21,16,13],1,1], //g#,c#,d,f
["E7sus",[52,54,57,62,18,20,17,16],1,1], //e,f#,a,d
["E7sus",[52,57,59,62,18,17,19,16],1,1], //e,a,b,d
["E7sus",[57,59,62,64,17,19,16,18],1,1], //a,b,d,e
["E7susb9",[52,53,57,62,18,13,17,16],1,1], //e,f,a,d
["Eb",[50,51,55,58,16,11,15,12],1,1], //d,eb,g,bb
["Eb/Bb",[58,62,63,67,12,16,11,15],1,1], //bb,d,eb,g
["Eb/Eb",[51,55,58,62,11,15,12,16],1,1], //eb,g,bb,d
["Eb/G",[55,58,62,63,15,12,16,11],1,1], //g,bb,d,eb
["Eb6/9",[55,58,60,65,15,12,14,13],1,1], //g,bb,c,f
["Eb6/9/G",[55,58,60,65,15,12,14,13],1,1], //g,bb,c,f
["Eb7",[55,60,61,65,15,14,9,13],1,1], //g,c,db,f
["Eb7#11",[51,55,57,61,11,15,17,9],1,1], //eb,g,a,db
["Eb7#11/Eb",[51,55,57,61,11,15,17,9],1,1], //eb,g,a,db
["Eb7#11/G",[55,57,61,63,15,17,9,11],1,1], //g,a,db,eb
["Eb7#9",[55,61,63,66,15,9,11,8],1,1], //g,db,eb,gb
["Eb7/Bb",[58,61,63,67,12,9,11,15],1,1], //bb,db,eb,g
["Eb7alt",[49,54,55,59,21,20,15,19],1,1], //c#,f#,g,b
["Eb7alt",[55,59,61,66,15,19,21,20],1,1], //g,b,c#,f#
["Eb7b13",[55,59,61,65,15,19,9,13],1,1], //g,b,db,f
["Eb7b9",[49,52,55,60,9,18,15,14],1,1], //db,e,g,c
["Eb7b9",[55,60,61,64,15,14,9,18],1,1], //g,c,db,e
["Eb7sus",[51,56,58,61,11,10,12,9],1,1], //eb,ab,bb,db
["Eb7sus",[56,58,61,63,10,12,9,11],1,1], //ab,bb,db,eb
["Eb7sus/Ab",[56,58,61,63,10,12,9,11],1,1], //ab,bb,db,eb
["Eb7susb9",[52,56,58,63,18,10,12,11],1,1], //e,ab,bb,eb
["Ebdim",[48,51,54,57,14,11,20,17],1,1], //c,eb,f#,a
["Ebdim",[51,54,57,60,11,20,17,14],1,1], //eb,f#,a,c
["Ebdim",[54,57,60,63,20,17,14,11],1,1], //f#,a,c,eb
["Ebdim",[57,60,63,66,17,14,11,20],1,1], //a,c,eb,f#
["Ebm(6/9)",[54,60,63,65,8,14,11,13],1,1], //gb,c,eb,f
["Ebm6",[54,58,60,63,8,12,14,11],1,1], //gb,bb,c,eb
["Ebm7",[49,51,54,58,9,11,8,12],1,1], //db,eb,gb,bb
["Ebm7",[49,53,54,58,9,13,8,12],1,1], //db,f,gb,bb
["Ebm7",[51,54,58,61,11,8,12,9],1,1], //eb,gb,bb,db
["Ebm7",[54,58,61,65,8,12,9,13],1,1], //gb,bb,db,f
["Ebm7#5",[51,54,59,61,11,8,19,9],1,1], //eb,gb,b,db
["Ebm7/Db",[49,51,54,58,9,11,8,12],1,1], //db,eb,gb,bb
["Ebm7b5",[51,56,57,61,11,10,17,9],1,1], //eb,ab,a,db
["Ebm7b5/A",[57,61,63,66,17,9,11,8],1,1], //a,db,eb,gb
["Ebm7b5/Gb",[54,57,61,63,8,17,9,11],1,1], //gb,a,db,eb
["EbMaj#11",[51,55,57,62,11,15,17,16],1,1], //eb,g,a,d
["EbMaj#5",[51,55,59,62,11,15,19,16],1,1], //eb,g,b,d
["EbMajb13",[51,55,59,62,11,15,19,16],1,1], //eb,g,b,d
["EbmMaj7",[50,51,54,58,16,11,8,12],1,1], //d,eb,gb,bb
["EbmMaj7",[51,54,58,62,11,8,12,16],1,1], //eb,gb,bb,d
["EbmMaj7",[54,58,62,65,8,12,16,13],1,1], //gb,bb,d,f
["EbmMaj7/D",[50,51,54,58,16,11,8,12],1,1], //d,eb,gb,bb
["Edim",[49,52,55,58,21,18,15,12],1,1], //c#,e,g,bb
["Edim",[52,55,58,61,18,15,12,21],1,1], //e,g,bb,c#
["Edim",[55,58,61,64,15,12,21,18],1,1], //g,bb,c#,e
["Edim",[58,61,64,67,12,21,18,15],1,1], //bb,c#,e,g
["Em(6/9)",[55,61,64,66,15,21,18,20],1,1], //g,c#,e,f#
["Em6",[52,55,59,61,18,15,19,21],1,1], //e,g,b,c#
["Em7",[50,52,55,59,16,18,15,19],1,1], //d,e,g,b
["Em7",[52,55,59,62,18,15,19,16],1,1], //e,g,b,d
["Em7",[55,59,62,66,15,19,16,20],1,1], //g,b,d,f#
["Em7#5",[52,55,60,62,18,15,14,16],1,1], //e,g,c,d
["Em7/D",[50,52,55,59,16,18,15,19],1,1], //d,e,g,b
["Em7/E",[52,55,59,62,18,15,19,16],1,1], //e,g,b,d
["Em7b5",[52,57,58,62,18,17,12,16],1,1], //e,a,bb,d
["Em7b5/Bb",[58,62,64,67,12,16,18,15],1,1], //bb,d,e,g
["Em7b5/G",[55,58,62,64,15,12,16,18],1,1], //g,bb,d,e
["EMaj#11",[52,56,58,63,18,22,24,23],1,1], //e,g#,a#,d#
["EMaj#5",[52,56,60,63,18,22,26,23],1,1], //e,g#,b#,d#
["EmMaj7",[51,52,55,59,23,18,15,19],1,1], //d#,e,g,b
["EmMaj7/D#",[51,52,55,59,23,18,15,19],1,1], //d#,e,g,b
["F",[52,53,57,60,18,13,17,14],1,1], //e,f,a,c
["F/A",[57,60,64,65,17,14,18,13],1,1],   //a,c,e,f
["F#",[53,54,58,61,13,20,24,21],1,1], //f,f#,a#,c#
["F#7",[52,56,58,63,18,22,24,23],1,1], //e,g#,a#,d#
["F#7#11",[54,58,60,64,20,24,14,18],1,1], //f#,a#,c,e
["F#7#9",[52,57,58,61,18,17,24,21],1,1], //e,a,a#,c#
["F#7/E",[52,54,58,61,18,20,24,21],1,1], //e,f#,a#,c#
["F#7alt",[52,57,58,62,18,17,12,16],1,1], //e,a,bb,d
["F#7b13",[52,56,58,62,18,22,24,16],1,1], //e,g#,a#,d
["F#7b13",[54,58,62,64,20,24,16,18],1,1], //f#,a#,d,e
["F#7b9",[52,55,58,63,18,15,24,23],1,1], //e,g,a#,d#
["F#7sus",[54,56,59,64,20,22,19,18],1,1], //f#,g#,b,e
["F#7sus",[54,59,61,64,20,19,21,18],1,1], //f#,b,c#,e
["F#7susb9",[54,55,59,64,20,15,19,18],1,1], //f#,g,b,e
["F#dim",[48,51,54,57,14,11,20,17],1,1], //c,eb,f#,a
["F#dim",[51,54,57,60,11,20,17,14],1,1], //eb,f#,a,c
["F#dim",[54,57,60,63,20,17,14,11],1,1], //f#,a,c,eb
["F#dim",[57,60,63,66,17,14,11,20],1,1], //a,c,eb,f#
["F#m(6/9)",[51,56,57,61,23,22,17,21],1,1], //d#,g#,a,c#
["F#m7",[52,56,57,61,18,22,17,21],1,1], //e,g#,a,c#
["F#m7#5",[54,57,60,64,20,17,14,18],1,1], //f#,a,c,e
["F#m7/E",[52,56,57,61,18,22,17,21],1,1], //e,g#,a,c#
["F#m7/F#",[54,57,61,64,20,17,21,18],1,1], //f#,a,c#,e
["F#m7b5",[52,54,57,60,18,20,17,14],1,1], //e,f#,a,c
["F#m7b5",[54,59,60,64,20,19,14,18],1,1], //f#,b,c,e
["F#m7b5/F#",[54,59,60,64,20,19,14,18],1,1], //f#,b,c,e
["F#mMaj7",[54,57,61,65,20,17,21,25],1,1], //f#,a,c#,e#
["F/F",[53,57,60,64,13,17,14,18],1,1], //f,a,c,e
["F6/9",[57,60,62,67,17,14,16,15],1,1], //a,c,d,g
["F7",[51,55,57,62,11,15,17,16],1,1], //eb,g,a,d
["F7",[57,62,63,67,17,16,11,15],1,1], //a,d,eb,g
["F7#11",[51,53,57,59,11,13,17,19],1,1], //eb,f,a,b
["F7#11",[53,57,59,63,13,17,19,11],1,1], //f,a,b,eb
["F7#9",[51,56,57,60,11,10,17,14],1,1], //eb,ab,a,c
["F7/A",[57,62,63,67,17,16,11,15],1,1], //a,d,eb,g
["F7/C",[48,51,53,57,14,11,13,17],1,1], //c,eb,f,a
["F7alt",[51,56,57,61,23,22,17,21],1,1], //d#,g#,a,c#
["F7alt",[57,61,63,68,17,21,23,22],1,1], //a,c#,d#,g#
["F7b13",[51,55,57,61,11,15,17,9],1,1], //eb,g,a,db
["F7b9",[51,54,57,62,11,20,17,16],1,1], //eb,f#,a,d
["F7sus",[53,58,60,63,13,12,14,11],1,1], //f,bb,c,eb
["F7sus/Bb",[58,60,63,65,12,14,11,13],1,1], //bb,c,eb,f
["F7sus/F",[53,58,60,63,13,12,14,11],1,1], //f,bb,c,eb
["F7susb9",[53,54,58,63,13,8,12,11],1,1], //f,gb,bb,eb
["Fdim",[50,53,56,59,16,13,10,19],1,1], //d,f,ab,b
["Fdim",[53,56,59,62,13,10,19,16],1,1], //f,ab,b,d
["Fdim",[56,59,62,65,10,19,16,13],1,1], //ab,b,d,f
["Fdim",[59,62,65,68,19,16,13,10],1,1], //b,d,f,ab
["Fm(6/9)",[50,55,56,60,16,15,10,14],1,1], //d,g,ab,c
["Fm6",[53,56,60,62,13,10,14,16],1,1], //f,ab,c,d
["Fm7",[51,55,56,60,11,15,10,14],1,1], //eb,g,ab,c
["Fm7",[56,60,63,67,10,14,11,15],1,1], //ab,c,eb,g
["Fm7#5",[53,56,61,63,13,10,9,11],1,1], //f,ab,db,eb
["Fm7/Eb",[51,55,56,60,11,15,10,14],1,1], //eb,g,ab,c
["Fm7/F",[53,56,60,63,13,10,14,11],1,1], //f,ab,c,eb
["Fm7b5",[51,53,56,59,11,13,10,19],1,1], //eb,f,ab,b
["Fm7b5",[53,58,59,63,13,12,7,11],1,1], //f,bb,cb,eb
["Fm7b5/Ab",[56,59,63,65,10,19,11,13],1,1], //ab,b,eb,f
["Fm7b5/B",[59,63,65,68,19,11,13,10],1,1], //b,eb,f,ab
["Fm7b5/Eb",[51,53,56,59,11,13,10,19],1,1], //eb,f,ab,b
["FMaj#11",[53,57,59,64,13,17,19,18],1,1], //f,a,b,e
["FMaj#5",[53,57,61,64,13,17,21,18],1,1], //f,a,c#,e
["FmMaj7",[52,53,56,60,18,13,10,14],1,1], //e,f,ab,c
["G",[54,55,59,62,20,15,19,16],1,1], //f#,g,b,d
["G#7",[54,58,60,65,20,24,14,13],1,1], //f#,a#,c,f
["G#7",[54,56,60,63,20,22,14,23],1,1], //f#,g#,c,d#
["G#7#11",[50,54,56,60,16,20,22,14],1,1], //d,f#,g#,c
["G#7b13",[54,58,60,64,20,24,14,18],1,1], //f#,a#,c,e
["G#m7",[54,58,59,63,20,24,19,23],1,1], //f#,a#,b,d#
["G#m7b5",[54,56,59,62,20,22,19,16],1,1], //f#,g#,b,d
["G#mMaj7",[55,56,59,63,15,22,19,23],1,1], //g,g#,b,d#
["G6/9",[52,57,59,62,18,17,19,16],1,1], //e,a,b,d
["G7",[53,57,59,64,13,17,19,18],1,1], //f,a,b,e
["G7/G",[55,59,62,65,15,19,16,13],1,1],   //g,b,d,f
["G7#11",[55,59,61,65,15,19,21,13],1,1], //g,b,c#,f
["G7#11/B",[59,61,65,67,19,21,13,15],1,1], //b,c#,f,g
["G7#9",[53,58,59,62,13,12,19,16],1,1], //f,bb,b,d
["G7/D",[50,53,55,59,16,13,15,19],1,1], //d,f,g,b
["G7alt",[53,58,59,63,13,12,7,11],1,1], //f,bb,cb,eb
["G7b13",[53,57,59,63,13,17,19,11],1,1], //f,a,b,eb
["G7b9",[53,56,59,64,13,10,19,18],1,1], //f,ab,b,e
["G7sus",[53,55,60,62,13,15,14,16],1,1], //f,g,c,d
["G7sus/C",[60,62,65,67,14,16,13,15],1,1], //c,d,f,g
["G7susb9",[55,56,60,65,15,10,14,13],1,1], //g,ab,c,f
["Gb",[53,54,58,61,13,8,12,9],1,1], //f,gb,bb,db
["Gb6/9",[51,56,58,61,11,10,12,9],1,1], //eb,ab,bb,db
["Gb7",[52,56,58,63,18,10,12,11],1,1], //e,ab,bb,eb
["Gb7b9",[52,55,58,63,18,15,12,11],1,1],   //e,g,bb,eb
["Gb7#11",[54,58,60,64,8,12,14,18],1,1], //gb,bb,c,e
["Gb7#11/Bb",[58,60,64,66,12,14,18,8],1,1], //bb,c,e,gb
["Gb7/Db",[49,52,54,58,9,18,8,12],1,1], //db,e,gb,bb
["Gb7b13",[52,56,58,62,18,10,12,16],1,1], //e,ab,bb,d
["Gb7sus",[54,56,59,64,8,10,19,18],1,1], //gb,ab,b,e
["Gb7sus",[54,59,61,64,8,19,9,18],1,1], //gb,b,db,e
["Gbdim",[54,57,60,63,20,17,14,11],1,1], //f#,a,c,eb
["Gbm6",[54,57,61,63,8,17,9,11],1,1], //gb,a,db,eb
["Gbm7",[52,54,57,61,18,8,17,9],1,1], //e,gb,a,db
["Gbm7/Db",[49,52,54,57,9,18,8,17],1,1], //db,e,gb,a
["Gbm7b5",[54,59,60,64,8,19,14,18],1,1], //gb,b,c,e
["GbMaj#11",[54,58,60,65,8,12,14,13],1,1], //gb,bb,c,f
["GbMaj#5",[54,58,62,65,8,12,16,13],1,1], //gb,bb,d,f
["GbmMaj7",[53,54,57,61,13,8,17,9],1,1], //f,gb,a,db
["Gdim",[49,52,55,58,21,18,15,12],1,1], //c#,e,g,bb
["Gdim",[52,55,58,61,18,15,12,21],1,1], //e,g,bb,c#
["Gdim",[55,58,61,64,15,12,21,18],1,1], //g,bb,c#,e
["Gdim",[58,61,64,67,12,21,18,15],1,1], //bb,c#,e,g
["Gm(6/9)",[52,57,58,62,18,17,12,16],1,1], //e,a,bb,d
["Gm5+",[55,58,63,65,15,12,23,13],1,1], //g,bb,d#,f
["Gm6",[55,58,62,64,15,12,16,18],1,1], //g,bb,d,e
["Gm7",[53,57,58,62,13,17,12,16],1,1], //f,a,bb,d
["Gm7#5",[55,58,63,65,15,12,11,13],1,1], //g,bb,eb,f
["Gm7/D",[50,53,55,58,16,13,15,12],1,1], //d,f,g,bb
["Gm7/F",[53,57,58,62,13,17,12,16],1,1], //f,a,bb,d
["Gm7b5",[53,55,58,61,13,15,12,9],1,1], //f,g,bb,db
["Gm7b5/Bb",[58,61,65,67,12,9,13,15],1,1], //bb,db,f,g
["Gm7b5/Db",[49,53,55,58,9,13,15,12],1,1], //db,f,g,bb
["GMaj#11",[55,59,61,66,15,19,21,20],1,1], //g,b,c#,f#
["GMaj#5",[55,59,63,63,15,19,23,23],1,1], //g,b,d#,d#
["GMaj#5",[55,59,63,66,15,19,23,20],1,1], //g,b,d#,f#
["GmMaj7",[54,55,58,62,20,15,12,16],1,1], //f#,g,bb,d
];


var tpc2note = [
[-1,"fbb"],
[0,"cbb"],
[1,"gbb"],
[2,"dbb"],
[3,"abb"],
[4,"ebb"],
[5,"bbb"],
[6,"fb"],
[7,"cb"],
[8,"gb"],
[9,"db"],
[10,"ab"],
[11,"eb"],
[12,"bb"],
[13,"f"],
[14,"c"],
[15,"g"],
[16,"d"],
[17,"a"],
[18,"e"],
[19,"b"],
[20,"f#"],
[21,"c#"],
[22,"g#"],
[23,"d#"],
[24,"a#"],
[25,"e#"],
[26,"b#"],
[27,"f##"],
[28,"c##"],
[29,"g##"],
[30,"d##"],
[31,"a##"],
[32,"e##"],
[33,"b##"],
];

// collect the chords entered in the xchord array
// the index i should parallel chordname
var xchord = new Array();
for(var i=0; i<512; i++) xchord[i] = [0,0,0,0,0,0,0,0];

var xchordname = new Array();
var cc = 0;

// collect the existing chord symbols and their segment.tick
var chordnames = new Array(512); 
// chordnames
// O: chordname
// 1: tick
// 2: xchordindex or -1 if none
for(var i=0; i<512; i++) chordnames[i]=new Array();
var count = 0;
var missing = 0;
// 1440 voor 3/4
// 1920 voor 4/4
var onemeasure = 1920;

console.log('MISSING CHORD SYMBOLS')

for(var i=0; i<512; i++) chordnames[i]=new Array();
var count = 0;
for (var track = 0; track <  1; ++track) {
	var segment = curScore.firstSegment();
    count = collectchordnames(segment, chordnames,count);
            
}
    // report missing chordnames to console log
    for (var c = 0; c < count; c++){
        var str = chordnames[c][0];

        // findstr return 1 or 0
        if (findstr (str,chords) == 0){
            console.log(str + " m" + (1 + chordnames[c][1]/onemeasure));
            missing++;
        }
    }
    
    console.log('missing ' + missing)
    if (missing > 0) Qt.quit();

    var cursor = curScore.newCursor();
    cursor.rewind(0); // beginning of score
    cursor.staffIdx = 1;

		//add a quarter note of time to the last measure
		//chordnames[count][1] = chordnames[count-1][1] + onemeasure;
        chordnames[count][1] = chordnames[count-1][1] + 480;
        
        setCursorToTime(cursor, chordnames[0][1]);
        var found = 0;

        // loop over chordnames & write chord to xchord if already there
        //console.log ('dcount = ' + count)
        for (var d = 0; d < count; d++){
            setCursorToTime(cursor, chordnames[d][1]);
            if (cursor.element && cursor.element.type === Element.CHORD){
                console.log('cursor.tick = ' + cursor.tick)
                //console.log('chordnames[cc][1] = ' + chordnames[cc][1])
                var s = "";
                // Now handle the note names on the main chord...
                var notes = cursor.element.notes;
                //var text = "[\"" + chordnames[cc++][0] + "\",[";
                var text = "[";
                for (var i = 0; i < notes.length; i++) {
                    
                    text += notes[i].pitch + ","
                    xchord[d][i] = notes[i].pitch;
                }
                for (var i = 0; i < notes.length ; i++) {
                    text += notes[i].tpc + ","
                    s += tpc2note[notes[i].tpc + 1][1] + ",";
                    xchord[d][i+4] = notes[i].tpc;
                }
                xchordname[d] = chordnames[d][0];
                console.log('xchord[d] = ' + chordnotes(xchord[d], tpc2note))
                
            } // end if (existing) CHORD
            else { // no chord yet at this symbol
            // console.log('d = ' + d);
            //console.log('chordnames[d][0] = ' + chordnames[d][0]);
                found = 0;
                for (var c = 0; c < chords.length && found == 0; c++){
                    cursor.staffIdx = 0;
                    //console.log('c = ' + c);
                    //console.log('chordnames[d][0] = ' + chordnames[d][0]);
                    //console.log('chords[c][0] = ' + chords[c][0]);

                    // chordname gevonden in chords
                    if (chordnames[d][0] == chords[c][0]){
                        cursor.staffIdx = 1;
                        // find the voicing closest to previous voicing used
                        // skip if this is the first chord symbol
                        if (d != 0){
                            // we expect max. 5 chords with this name
                            var best = [0,0,0,0,0];
                            var thisname = chordnames[d][0];
                            
                            console.log('thisname = ' + thisname + " m" + (1 + cursor.tick/onemeasure));
                            
                            var i = 0;
                            while ((c+i < chords.length) && chords[c+i][0] == thisname){
                                // the current chord in the table
                                var chord1 = chords[c+i][1];
                                // the previous chord in the score
                                var chord2 = xchord[d-1];
                                console.log('chord1 = ' + chordnotes(chord1,tpc2note));
                                console.log('chord2 = ' + chordnotes(xchord[d-1],tpc2note));
                                //console.log('xchordname[d-1] = ' + xchordname[d-1]);
                                console.log('chordnames[d-1] = ' + chordnames[d-1][0]);
                                // calculate the cumul distance between pitches chord1-chord2
                                for (var e=0;e< 4; e++){
                                    best[i] = best[i] + abs(chord1[e] - chord2[e])
                                }
                                // best[i] is the distance for this pair
                                console.log('best[i] = ' + best[i]);
                                i++;
                            }
                            var bestmin = 1000;
                            var besti = 0;
                            for (var j=0;j< i; j++){
                                if (bestmin > best[j]) {
                                    bestmin = best[j];
                                    besti = j;
                                }
                            }
                            c += besti;
                            console.log('besti = ' + besti);
                            console.log('besti chord = ' + chords[c][0]);
                            console.log ('c = ' + c)   
                        }
                        var chord = chords[c][1];
                        // store this chord in xchord; will be 'previous' on the next round
                        xchord[d] = chord;
                        console.log('chordname = ' + chordnames[d][0]);
                        var duration = chordnames[d+1][1] - chordnames[d][1];
                        if(duration > 1920) duration = 1920;
                        //console.log('chords[c][0] = ' + chords[c][0]);
                        //console.log('duration = ' + duration);
                        
                        console.log('adding chord ' +  chordnotes(chord, tpc2note))
                        addChord(cursor, chord, duration);

                        found = 1;
                    } // end of chordname found
                    //console.log('found = ' + found);
                } // end of this chordname
                // not used anymore
                if (found == 0 && chordnames[d][0] != ""){
                    console.log('d = ' + d);
                    console.log('tick = ' + chordnames[d][1]);
                    console.log(chordnames[d][0] + ' measure: ' + chordnames[d][1]/onemeasure + ' not found');
                    Qt.quit();
                    // d = -1;
                }
            } // end of else: no chord yet
            
		} // end of chordname loop
		Qt.quit();
    } // end onRun
}
